/**
 * Copyright ProjectNya ( http://wonderfl.net/user/ProjectNya )
 * MIT License ( http://www.opensource.org/licenses/mit-license.php )
 * Downloaded from: http://wonderfl.net/c/z5Bs
 */

////////////////////////////////////////////////////////////////////////////////
// [AS3.0] カラー補正クラスに挑戦！ (3)
// http://www.project-nya.jp/modules/weblog/details.php?blog_id=1089
////////////////////////////////////////////////////////////////////////////////

package {

	import flash.display.Sprite;
	import flash.display.StageScaleMode;
	import flash.display.StageAlign;
	import flash.events.Event;
	import flash.events.MouseEvent;

	[SWF(backgroundColor="#FFFFFF", width="465", height="465", frameRate="30")]

	public class Main extends Sprite {
		private static var basePath:String = "http://www.project-nya.jp/images/flash/";
		private var colorMatrixManager:ColorMatrixManager;
		private var loader:PhotoLoader;
		private var photoList:Array;
		private var photoID:uint = 0;
		private var menu:Menu;
		private var slider1:Slider;
		private var slider2:Slider;
		private var slider3:Slider;
		private var wheel:Wheel;

		public function Main() {
			Wonderfl.capture_delay(4);
			init();
		}

		private function init():void {
			loader = new PhotoLoader();
			loader.x = 72;
			loader.y = 30;
			addChild(loader);
			loader.addEventListener(PhotoLoader.INIT, initialize, false, 0, true);
			loader.addEventListener(PhotoLoader.COMPLETE, complete, false, 0, true);
			photoList = new Array();
			photoList.push({label: "photo1", path: "colorManager_photo1.jpg"});
			photoList.push({label: "photo2", path: "colorManager_photo2.jpg"});
			photoList.push({label: "photo3", path: "colorManager_photo3.jpg"});
			photoList.push({label: "photo4", path: "colorManager_photo4.jpg"});
			menu = new Menu();
			addChild(menu);
			menu.x = 82;
			menu.y = 5;
			menu.init({label: "photo"});
			menu.dataProvider = photoList;
			menu.initialize(photoID);
			menu.addEventListener(CompoEvent.SELECT, select, false, 0, true);
			load(photoID);
			slider1 = new Slider();
			slider1.x = 82;
			slider1.y = 280;
			addChild(slider1);
			slider1.init({label: "brightness", width: 200, grid: 10, min: -100, max: 100, init: 0});
			slider1.addEventListener(CompoEvent.CHANGE, change1, false, 0, true);
			slider2 = new Slider();
			slider2.x = 82;
			slider2.y = 340;
			addChild(slider2);
			slider2.init({label: "contrast", width: 200, grid: 10, min: 0, max: 200, init: 100});
			slider2.addEventListener(CompoEvent.CHANGE, change2, false, 0, true);
			slider3 = new Slider();
			slider3.x = 82;
			slider3.y = 400;
			addChild(slider3);
			slider3.init({label: "saturation", width: 200, grid: 10, min: 0, max: 200, init: 100});
			slider3.addEventListener(CompoEvent.CHANGE, change3, false, 0, true);
			wheel = new Wheel();
			wheel.x = 302;
			wheel.y = 280;
			addChild(wheel);
			wheel.init({label: "hue", angle: 0});
			wheel.addEventListener(CompoEvent.CHANGE, change4, false, 0, true);
			var resetBtn:Btn = new Btn();
			resetBtn.x = 352;
			resetBtn.y = 440;
			addChild(resetBtn);
			resetBtn.init({label: "reset"});
			resetBtn.addEventListener(MouseEvent.CLICK, reset, false, 0, true);
		}
		private function select(evt:CompoEvent):void {
			loader.unload();
			photoID = evt.value;
			load(photoID);
		}
		private function load(id:uint):void {
			var filePath:String = photoList[id].path;
			loader.load(basePath + filePath);
		}
		private function initialize(evt:Event):void {
		}
		private function complete(evt:Event):void {
			colorMatrixManager = new ColorMatrixManager(PhotoLoader(evt.target));
			reset();
		}
		private function change1(evt:CompoEvent):void {
			colorMatrixManager.brightness(evt.value);
		}
		private function change2(evt:CompoEvent):void {
			colorMatrixManager.contrast(evt.value);
		}
		private function change3(evt:CompoEvent):void {
			colorMatrixManager.saturation(evt.value);
		}
		private function change4(evt:CompoEvent):void {
			colorMatrixManager.hue(evt.value);
		}
		private function reset(evt:MouseEvent = null):void {
			slider1.reset();
			slider2.reset();
			slider3.reset();
			wheel.reset();
			colorMatrixManager.reset();
		}

	}

}


//////////////////////////////////////////////////
//	ColorMatrixManager(カラー補正)クラス
//////////////////////////////////////////////////

import flash.display.DisplayObject;
import flash.filters.ColorMatrixFilter;
//import com.quasimondo.geom.ColorMatrix;

class ColorMatrixManager {
	private var target:DisplayObject;
	private var colorMatrixFilter:ColorMatrixFilter;
	//private var colorMatrix:ColorMatrix;
	private var colorMatrix:Array;
	private var _brightness:Number = 0;
	private var _contrast:Number = 100;
	private var _saturation:Number = 100;
	private var _hue:Number = 0;
	private static var lr:Number = 0.212671;
	private static var lg:Number = 0.71516;
	private static var lb:Number = 0.072169;
	private static var radian:Number = Math.PI/180;

	public function ColorMatrixManager(tg:DisplayObject) {
		target = tg;
		colorMatrixFilter = new ColorMatrixFilter();
		//colorMatrix = new ColorMatrix();
		colorMatrix = new Array();
	}

	public function brightness(param:Number):void {
		_brightness = param;
		setColorMatrix();
	}
	public function contrast(param:Number):void {
		_contrast = param;
		setColorMatrix();
	}
	public function saturation(param:Number):void {
		_saturation = param;
		setColorMatrix();
	}
	public function hue(param:Number):void {
		_hue = param;
		setColorMatrix();
	}
	private function setColorMatrix():void {
		//colorMatrix.reset();
		//colorMatrix.adjustBrightness(_brightness*2.55);
		//colorMatrix.adjustContrast(_contrast*0.01 - 1);
		//colorMatrix.adjustSaturation(_saturation*0.01);
		//colorMatrix.adjustHue(_hue);
		//colorMatrixFilter.matrix = colorMatrix.matrix;
		colorMatrix = [1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0];
		adjustBrightness(_brightness*2.55);
		adjustContrast(_contrast*0.01 - 1);
		adjustSaturation(_saturation*0.01);
		adjustHue(_hue);
		colorMatrixFilter.matrix = colorMatrix;
		target.filters = [colorMatrixFilter];
	}
	public function reset():void {
		_brightness = 0;
		_contrast = 100;
		_saturation = 100;
		_hue = 0;
		setColorMatrix();
	}
	private function concat(matrix:Array):void {
		var list:Array = new Array();
		var n:uint = 0;
		for (var py:uint = 0; py < 4; py++) {
			for (var px:uint = 0; px < 5; px++) {
				list[uint(n + px)] = Number(matrix[n])*Number(colorMatrix[px]) + Number(matrix[uint(n + 1)])*Number(colorMatrix[uint(px + 5)])
					+ Number(matrix[uint(n + 2)])*Number(colorMatrix[uint(px + 10)]) + Number(matrix[uint(n + 3)])*Number(colorMatrix[uint(px + 15)])
					+ (px == 4 ? Number(matrix[uint(n + 4)]) : 0);
			}
			n += 5;
		}
		colorMatrix = list;
	}
	private function adjustBrightness(s:Number):void {
		var r:Number;
		var g:Number;
		var b:Number;
		r = g = b = s;
		concat([1, 0, 0, 0, r, 0, 1, 0, 0, g, 0, 0, 1, 0, b, 0, 0, 0, 1, 0]);
	}
	private function adjustContrast(s:Number):void {
		var r:Number;
		var g:Number;
		var b:Number;
		r = g = b = s + 1;
		concat([r, 0, 0, 0, (128 * (1 - r)), 0, g, 0, 0, (128 * (1 - g)), 0, 0, b, 0, (128 * (1 - b)), 0, 0, 0, 1, 0]);
	}
	private function adjustSaturation(s:Number):void {
		var sv:Number = 1 - s;
		var r:Number = sv*lr;
		var g:Number = sv*lg;
		var b:Number = sv*lb;
		concat([(r + s), g, b, 0, 0, r, (g + s), b, 0, 0, r, g, (b + s), 0, 0, 0, 0, 0, 1, 0]);
	}
	private function adjustHue(a:Number):void {
		var cos:Number = Math.cos(a*radian);
		var sin:Number = Math.sin(a*radian);
		concat([lr + cos*(1 - lr) - sin*lr, lg - cos*lg - sin*lg, lb - cos*lb + sin*(1 - lb), 0, 0, lr - cos*lr + sin*0.143, lg + cos*(1 - lg) + sin*0.14,  lb - cos*lb - sin*0.283, 0, 0, lr - cos*lr - sin*(1 - lr), lg - cos*lg + sin*lg, lb + cos*(1 - lb) + sin*lb, 0, 0, 0, 0, 0, 1, 0]);
	}

}


//////////////////////////////////////////////////
//	PhotoLoaderクラス
//////////////////////////////////////////////////

import flash.display.Sprite;
import flash.display.Loader;
import flash.display.LoaderInfo;
import flash.net.URLRequest;
import flash.events.Event;
import flash.events.IOErrorEvent;
import flash.events.HTTPStatusEvent;
import flash.events.SecurityErrorEvent;
import flash.display.Bitmap;
import flash.system.LoaderContext;

class PhotoLoader extends Sprite {
	private var loader:Loader;
	private var info:LoaderInfo;
	public var content:*;
	private var smoothing:Boolean;
	public static const IO_ERROR:String = IOErrorEvent.IO_ERROR;
	public static const HTTP_STATUS:String = HTTPStatusEvent.HTTP_STATUS;
	public static const SECURITY_ERROR:String = SecurityErrorEvent.SECURITY_ERROR;
	public static const INIT:String = Event.INIT;
	public static const COMPLETE:String = Event.COMPLETE;

	public function PhotoLoader() {
		loader = new Loader();
		info = loader.contentLoaderInfo;
	}

	public function load(file:String, s:Boolean = false):void {
		smoothing = s;
		info.addEventListener(IOErrorEvent.IO_ERROR, ioerror, false, 0, true);
		info.addEventListener(HTTPStatusEvent.HTTP_STATUS, httpstatus, false, 0, true);
		info.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityerror, false, 0, true);
		info.addEventListener(Event.INIT, initialize, false, 0, true);
		info.addEventListener(Event.COMPLETE, complete, false, 0, true);
		try {
			loader.load(new URLRequest(file), new LoaderContext(true));
		} catch (err:Error) {
			trace(err.message);
		}
	}
	public function unload():void {
		loader.unload();
	}
	private function ioerror(evt:IOErrorEvent):void {
		loader.unload();
		dispatchEvent(new Event(PhotoLoader.IO_ERROR));
	}
	private function httpstatus(evt:HTTPStatusEvent):void {
		dispatchEvent(new Event(PhotoLoader.HTTP_STATUS));
	}
	private function securityerror(evt:SecurityErrorEvent):void {
		dispatchEvent(new Event(PhotoLoader.SECURITY_ERROR));
	}
	private function initialize(evt:Event):void {
		if (smoothing) {
			content = Bitmap(info.content);
			content.smoothing = true;
		} else {
			content = info.content;
		}
		dispatchEvent(new Event(PhotoLoader.INIT));
	}
	private function complete(evt:Event):void {
		info.removeEventListener(IOErrorEvent.IO_ERROR, ioerror);
		info.removeEventListener(HTTPStatusEvent.HTTP_STATUS, httpstatus);
		info.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, securityerror);
		info.removeEventListener(Event.INIT, initialize);
		info.removeEventListener(Event.COMPLETE, complete);
		addChild(loader);
		dispatchEvent(new Event(PhotoLoader.COMPLETE));
	}
	public function centerize():void {
		content.x = -uint(content.width*0.5);
		content.y = -uint(content.height*0.5);
	}

}


//////////////////////////////////////////////////
//	Menuクラス
//////////////////////////////////////////////////

import flash.display.Sprite;
import flash.display.Shape;
import flash.text.TextField;
import flash.text.TextFieldType;
import flash.text.AntiAliasType;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;
import flash.events.MouseEvent;
import flash.geom.ColorTransform;

class Menu extends Sprite {
	public var id:uint;
	private var selectedID:uint;
	private var selectedItem:TextField;
	private var tab:Sprite;
	private var base:Shape;
	private var txt:TextField;
	private var label:String = "";
	private static var fontType:String = "_ゴシック";
	private var _width:uint = 60;
	private static var _height:uint = 20;
	private static var tHeight:uint = 20;
	private static var bColor:uint = 0xFFFFFF;
	private static var cColor:uint = 0x3165B5;
	private static var upColor:uint = 0x000000;
	private static var overColor:uint = 0xFFFFFF;
	private static var offColor:uint = 0x999999;
	private static var bColorTrans:ColorTransform;
	private static var cColorTrans:ColorTransform;
	private var child:MenuChild;
	private var dataList:Array;
	private var _enabled:Boolean = true;
	private var _selected:Boolean = false;

	public function Menu() {
	}

	public function init(option:Object):void {
		if (option.id != undefined) id = option.id;
		if (option.label != undefined) label = option.label;
		if (option.width != undefined) _width = option.width;
		draw();
	}
	private function draw():void {
		bColorTrans = new ColorTransform();
		bColorTrans.color = bColor;
		cColorTrans = new ColorTransform();
		cColorTrans.color = cColor;
		selectedItem = new TextField();
		tab = new Sprite();
		base = new Shape();
		txt = new TextField();
		addChild(selectedItem);
		addChild(tab);
		tab.addChild(base);
		tab.addChild(txt);
		selectedItem.x = _width + 5;
		selectedItem.y = 1;
		selectedItem.width = _width;
		selectedItem.height = _height - 1;
		selectedItem.type = TextFieldType.DYNAMIC;
		selectedItem.selectable = false;
		//selectedItem.embedFonts = true;
		//selectedItem.antiAliasType = AntiAliasType.ADVANCED;
		var tfl:TextFormat = new TextFormat();
		tfl.font = fontType;
		tfl.size = 12;
		tfl.align = TextFormatAlign.LEFT;
		selectedItem.defaultTextFormat = tfl;
		createBox(base, _width, _height);
		txt.y = 1;
		txt.width = _width;
		txt.height = _height - 1;
		txt.type = TextFieldType.DYNAMIC;
		txt.selectable = false;
		//txt.embedFonts = true;
		//txt.antiAliasType = AntiAliasType.ADVANCED;
		var tf:TextFormat = new TextFormat();
		tf.font = fontType;
		tf.size = 12;
		tf.align = TextFormatAlign.CENTER;
		txt.defaultTextFormat = tf;
		txt.text = label;
		_up();
		enabled = true;
		tab.mouseChildren = false;
	}
	private function rollOver(evt:MouseEvent):void {
		_over();
	}
	private function rollOut(evt:MouseEvent):void {
		up();
	}
	private function press(evt:MouseEvent):void {
		_over();
	}
	private function release(evt:MouseEvent):void {
		_over();
	}
	private function click(evt:MouseEvent):void {
		_over();
		child.opencloseMenu();
	}
	private function up():void {
		if (_selected) {
			_over();
		} else {
			_up();
		}
	}
	private function _up():void {
		txt.textColor = upColor;
		base.transform.colorTransform = bColorTrans;
	}
	private function _over():void {
		txt.textColor = overColor;
		base.transform.colorTransform = cColorTrans;
	}
	private function _off():void {
		txt.textColor = offColor;
		base.transform.colorTransform = bColorTrans;
	}
	public function set dataProvider(list:Array):void {
		dataList = list;
		if (dataList.length > 0) addChildren();
	}
	private function addChildren():void {
		child = new MenuChild(dataList, this);
		addChild(child);
		child.y = tHeight;
		child.visible = false;
		child.addEventListener(MouseEvent.CLICK, select, false, 0, true);
	}
	private function mouseDown(evt:MouseEvent):void {
		if (!hitTestPoint(stage.mouseX, stage.mouseY, true)) {
		child.closeMenu();
		}
	}
	public function initialize(param:uint):void {
		selectedID = param;
		child.selectItem(selectedID);
		show();
	}
	private function select(evt:MouseEvent):void {
		selectedID = evt.target.id;
		var e:CompoEvent = new CompoEvent(CompoEvent.SELECT, selectedID);
		dispatchEvent(e);
		show();
	}
	public function get selected():Boolean {
		return _selected;
	}
	public function set selected(param:Boolean):void {
		_selected = param;
		if (_enabled) up();
		show();
	}
	public function get enabled():Boolean {
		return _enabled;
	}
	public function set enabled(param:Boolean):void {
		_enabled = param;
		tab.buttonMode = _enabled;
		tab.mouseEnabled = _enabled;
		tab.useHandCursor = _enabled;
		if (_enabled) {
			_up();
			tab.addEventListener(MouseEvent.MOUSE_OVER, rollOver, false, 0, true);
			tab.addEventListener(MouseEvent.MOUSE_OUT, rollOut, false, 0, true);
			tab.addEventListener(MouseEvent.MOUSE_DOWN, press, false, 0, true);
			tab.addEventListener(MouseEvent.MOUSE_UP, release, false, 0, true);
			tab.addEventListener(MouseEvent.CLICK, click, false, 0, true);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, mouseDown, false, 0, true);
		} else {
			_off();
			tab.removeEventListener(MouseEvent.MOUSE_OVER, rollOver);
			tab.removeEventListener(MouseEvent.MOUSE_OUT, rollOut);
			tab.removeEventListener(MouseEvent.MOUSE_DOWN, press);
			tab.removeEventListener(MouseEvent.MOUSE_UP, release);
			tab.removeEventListener(MouseEvent.CLICK, click);
			stage.removeEventListener(MouseEvent.MOUSE_DOWN, mouseDown);
		}
	}
	private function createBox(target:Shape, w:uint, h:uint):void {
		target.graphics.clear();
		target.graphics.beginFill(bColor);
		target.graphics.drawRect(0, 0, w, h);
		target.graphics.endFill();
	}
	private function show():void {
		var str:String = dataList[selectedID].label;
		selectedItem.text = String(str);
	}

}


import flash.display.Sprite;
import flash.display.Shape;
import flash.events.MouseEvent;
import flash.filters.DropShadowFilter;

class MenuChild extends Sprite {
	private var _width:uint = 100;
	private var _height:uint;
	private static var tHeight:uint = 20;
	private static var bColor:uint = 0xFFFFFF;
	private static var sColor:uint = 0x000000;
	private var dataList:Array;
	private var max:uint;
	private var itemList:Array;
	private var maxWidth:uint = 0;
	private var back:Shape;
	private var shade:DropShadowFilter;
	private var menu:Menu;
	private var opened:Boolean = false;
	private var selectedID:uint;

	public function MenuChild(list:Array, m:Menu) {
		dataList = list;
		max = dataList.length;
		_height = tHeight*max;
		menu = m;
		init();
	}

	private function init():void {
		back = new Shape();
		addChild(back);
		shade = new DropShadowFilter(1, 90, sColor, 0.5, 4, 4, 2, 3, false, false);
		back.filters = [shade];
		itemList = new Array();
		for (var n:uint = 0; n < max; n++) {
			var item:MenuItem = new MenuItem({id: n, label: dataList[n].label});
			addChild(item);
			item.y = tHeight*n;
			itemList.push(item);
			item.addEventListener(MouseEvent.CLICK, select, false, 0, true);
			resizeWidth(item);
		}
	}
	private function select(evt:MouseEvent):void {
		closeMenu();
		selectedID = MenuItem(evt.currentTarget).id;
		checkItem();
	}
	public function selectItem(id:uint):void {
		selectedID = id;
		checkItem();
	}
	public function opencloseMenu():void {
		if (!opened) {
			openMenu();
		} else {
			closeMenu();
		}
	}
	private function openMenu():void {
		opened = true;
		visible = true;
		menu.selected = true;
	}
	public function closeMenu():void {
		opened = false;
		visible = false;
		menu.selected = false;
	}
	private function checkItem():void {
		for (var n:uint = 0; n < itemList.length; n++) {
			var item:MenuItem = itemList[n];
			if (n == selectedID) {
				item.selected = true;
			} else {
				item.selected = false;
			}
		}
	}
	private function resizeWidth(item:MenuItem):void {
		if (item._width > maxWidth) maxWidth = item._width;
		if (itemList.length >= max) resize();
	}
	private function resize():void {
		_width = maxWidth;
		createBox(back, _width, _height);
		for (var n:uint = 0; n < max; n++) {
			var item:MenuItem = itemList[n];
			item._width = maxWidth;
			item.txt.width = item._width - 20;
			createBox(item.base, item._width, item._height);
		}
	}
	private function createBox(target:Shape, w:uint, h:uint):void {
		target.graphics.clear();
		target.graphics.beginFill(bColor);
		target.graphics.drawRect(0, 0, w, h);
		target.graphics.endFill();
	}

}


import flash.display.Sprite;
import flash.display.Shape;
import flash.text.TextField;
import flash.text.TextFieldType;
import flash.text.AntiAliasType;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;
import flash.events.MouseEvent;
import flash.geom.ColorTransform;

class MenuItem extends Sprite {
	public var id:uint;
	private var item:Sprite;
	public var base:Shape;
	public var txt:TextField;
	private var check:TextField;
	public var _width:uint = 100;
	public var _height:uint = 20;
	private var label:String = "";
	private static var fontType:String = "_ゴシック";
	private var mark:String = String.fromCharCode(10003);
	private static var checkType:String = "_ゴシック";
	private static var bColor:uint = 0xFFFFFF;
	private static var cColor:uint = 0x3165B5;
	private static var upColor:uint = 0x000000;
	private static var overColor:uint = 0xFFFFFF;
	private static var bColorTrans:ColorTransform;
	private static var cColorTrans:ColorTransform;
	private var _selected:Boolean = false;

	public function MenuItem(option:Object) {
		if (option.id != undefined) id = option.id;
		if (option.label) label = option.label;
		init();
	}

	private function init():void {
		bColorTrans = new ColorTransform();
		bColorTrans.color = bColor;
		cColorTrans = new ColorTransform();
		cColorTrans.color = cColor;
		item = new Sprite();
		base = new Shape();
		txt = new TextField();
		check = new TextField();
		addChild(item);
		item.addChild(base);
		item.addChild(txt);
		item.addChild(check);
		txt.x = 20;
		txt.y = 1;
		txt.width = _width;
		txt.height = _height - 1;
		txt.type = TextFieldType.DYNAMIC;
		txt.selectable = false;
		//txt.embedFonts = true;
		//txt.antiAliasType = AntiAliasType.ADVANCED;
		var tf:TextFormat = new TextFormat();
		tf.font = fontType;
		tf.size = 12;
		tf.align = TextFormatAlign.LEFT;
		txt.defaultTextFormat = tf;
		txt.text = label;
		_width = txt.textWidth + 35;
		check.x = 3;
		check.y = -1;
		check.width = 14;
		check.height = 22;
		check.type = TextFieldType.DYNAMIC;
		check.selectable = false;
		//check.embedFonts = true;
		//check.antiAliasType = AntiAliasType.ADVANCED;
		var tfc:TextFormat = new TextFormat();
		tfc.font = checkType;
		tfc.size = 12;
		tfc.align = TextFormatAlign.CENTER;
		check.defaultTextFormat = tfc;
		check.text = mark;
		check.visible = false;
		buttonMode = true;
		mouseEnabled = true;
		useHandCursor = true;
		_up();
		mouseChildren = false;
		addEventListener(MouseEvent.MOUSE_OVER, rollOver, false, 0, true);
		addEventListener(MouseEvent.MOUSE_OUT, rollOut, false, 0, true);
		addEventListener(MouseEvent.MOUSE_DOWN, press, false, 0, true);
		addEventListener(MouseEvent.MOUSE_UP, release, false, 0, true);
		addEventListener(MouseEvent.CLICK, click, false, 0, true);
	}
	private function rollOver(evt:MouseEvent):void {
		_over();
	}
	private function rollOut(evt:MouseEvent):void {
		_up();
	}
	private function press(evt:MouseEvent):void {
		_over();
	}
	private function release(evt:MouseEvent):void {
		_up();
	}
	private function click(evt:MouseEvent):void {
		_up();
	}
	private function _up():void {
		txt.textColor = upColor;
		check.textColor = upColor;
		base.transform.colorTransform = bColorTrans;
	}
	private function _over():void {
		txt.textColor = overColor;
		check.textColor = overColor;
		base.transform.colorTransform = cColorTrans;
	}
	public function get selected():Boolean {
		return _selected;
	}
	public function set selected(param:Boolean):void {
		_selected = param;
		check.visible = _selected;
	}

}


//////////////////////////////////////////////////
//	 Sliderクラス
//////////////////////////////////////////////////

import flash.display.Sprite;
import flash.display.Shape;
import flash.text.TextField;
import flash.text.TextFieldType;
import flash.text.AntiAliasType;
import flash.text.TextFieldAutoSize;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;
import flash.filters.DropShadowFilter;
import flash.filters.GlowFilter;
import flash.geom.Rectangle;
import flash.geom.Matrix;
import flash.display.GradientType;
import flash.display.SpreadMethod;
import flash.display.InterpolationMethod;
import flash.events.Event;
import flash.events.MouseEvent;

class Slider extends Sprite {
	private var hole:Shape;
	private var line:Sprite;
	private var thumb:Sprite;
	private var light:Shape;
	private var shade:Shape;
	private var base:Shape;
	private var title:TextField;
	private var txt:TextField;
	private var label:String = "";
	private static var fontType:String = "_ゴシック";
	private var _width:uint = 100;
	private static var tHeight:uint = 20;
	private static var bHeight:uint = 30;
	private var grid:uint = 5;
	private static var bColor:uint = 0xFFFFFF;
	private static var tColor:uint = 0x666666;
	private static var gColor:uint = 0x999999;
	private static var mColor:uint = 0x333333;
	private static var bgColor:uint = 0x0099FF;
	private static var sColor:uint = 0x000000;
	private static var offColor:uint = 0x999999;
	private var min:Number = 0;
	private var max:Number = 100;
	private var initValue:Number = 0;
	private var blueGlow:GlowFilter;
	private var shadeDrop:DropShadowFilter;
	private var value:Number;
	private var _enabled:Boolean = true;

	public function Slider() {
	}

	public function init(option:Object):void {
		if (option.label != undefined) label = option.label;
		if (option.width != undefined) _width = option.width;
		if (option.min != undefined) min = option.min;
		if (option.max != undefined) max = option.max;
		if (option.grid != undefined) grid = option.grid;
		if (option.init != undefined) initValue = option.init;
		draw();
	}
	private function draw():void {
		shadeDrop = new DropShadowFilter(1, 90, sColor, 0.5, 4, 4, 2, 3, false, false);
		blueGlow = new GlowFilter(bgColor, 0.6, 5, 5, 2, 3, false, true);
		hole = new Shape();
		line = new Sprite();
		title = new TextField();
		txt = new TextField();
		thumb = new Sprite();
		shade = new Shape();
		light = new Shape();
		base = new Shape();
		addChild(hole);
		addChild(line);
		addChild(title);
		addChild(txt);
		addChild(thumb);
		thumb.addChild(shade);
		thumb.addChild(light);
		thumb.addChild(base);
		hole.y = bHeight;
		createGradientHole(hole, _width, 3);
		line.y = bHeight;
		createGrid(line);
		title.height = tHeight-1;
		title.type = TextFieldType.DYNAMIC;
		title.selectable = false;
		//title.embedFonts = true;
		//title.antiAliasType = AntiAliasType.ADVANCED;
		title.textColor = tColor;
		title.autoSize = TextFieldAutoSize.LEFT;
		var tfl:TextFormat = new TextFormat();
		tfl.font = fontType;
		tfl.size = 12;
		tfl.align = TextFormatAlign.LEFT;
		title.defaultTextFormat = tfl;
		title.text = label;
		//txt.x = title.textWidth;
		txt.x = 150;
		txt.width = 50;
		txt.height = tHeight-1;
		txt.selectable = false;
		//txt.embedFonts = true;
		//txt.antiAliasType = AntiAliasType.ADVANCED;
		var tfr:TextFormat = new TextFormat();
		tfr.font = fontType;
		tfr.size = 12;
		tfr.align = TextFormatAlign.RIGHT;
		txt.defaultTextFormat = tfr;
		reset();
		thumb.y = bHeight;
		createThumb(shade, 8, 20, 12, sColor);
		shade.filters = [shadeDrop];
		createThumb(light, 8, 20, 12, bgColor);
		light.filters = [blueGlow];
		createThumb(base, 8, 20, 12, bColor);
		_up();
		enabled = true;
		thumb.mouseChildren = false;
	}
	private function rollOver(evt:MouseEvent):void {
		_over();
	}
	private function rollOut(evt:MouseEvent):void {
		_up();
	}
	private function press(evt:MouseEvent):void {
		_down();
		var rect:Rectangle = new Rectangle(0, bHeight, _width, 0);
		thumb.startDrag(false, rect);
		thumb.addEventListener(MouseEvent.MOUSE_UP, release, false, 0, true);
		stage.addEventListener(MouseEvent.MOUSE_UP, release, false, 0, true);
		stage.addEventListener(Event.MOUSE_LEAVE, leave, false, 0, true);
		thumb.addEventListener(Event.ENTER_FRAME, change, false, 0, true);
	}
	private function release(evt:MouseEvent):void {
		_up();
		thumb.stopDrag();
		checkValue();
		var e:CompoEvent = new CompoEvent(CompoEvent.SELECT, value);
		dispatchEvent(e);
		thumb.removeEventListener(MouseEvent.MOUSE_UP, release);
		stage.removeEventListener(MouseEvent.MOUSE_UP, release);
		stage.removeEventListener(Event.MOUSE_LEAVE, leave);
		thumb.removeEventListener(Event.ENTER_FRAME, change);
	}
	private function leave(evt:Event):void {
		_up();
		thumb.stopDrag();
		checkValue();
		var e:CompoEvent = new CompoEvent(CompoEvent.SELECT, value);
		dispatchEvent(e);
		thumb.removeEventListener(MouseEvent.MOUSE_UP, release);
		stage.removeEventListener(MouseEvent.MOUSE_UP, release);
		stage.removeEventListener(Event.MOUSE_LEAVE, leave);
		thumb.removeEventListener(Event.ENTER_FRAME, change);
	}
	private function _up():void {
		light.visible = false;
	}
	private function _over():void {
		light.visible = true;
	}
	private function _down():void {
		light.visible = true;
	}
	private function _off():void {
		light.visible = false;
		txt.textColor = offColor;
	}
	private function change(evt:Event):void {
		_down();
		checkValue();
		var e:CompoEvent = new CompoEvent(CompoEvent.CHANGE, value);
		dispatchEvent(e);
	}
	private function checkValue():void {
		value = min + Math.round(thumb.x/_width*(max-min));
		txt.text = String(value);
	}
	public function get enabled():Boolean {
		return _enabled;
	}
	public function set enabled(param:Boolean):void {
		_enabled = param;
		if (!_enabled) _off();
		thumb.buttonMode = _enabled;
		thumb.mouseEnabled = _enabled;
		thumb.useHandCursor = _enabled;
		if (_enabled) {
			thumb.addEventListener(MouseEvent.MOUSE_OVER, rollOver, false, 0, true);
			thumb.addEventListener(MouseEvent.MOUSE_OUT, rollOut, false, 0, true);
			thumb.addEventListener(MouseEvent.MOUSE_DOWN, press, false, 0, true);
			thumb.addEventListener(MouseEvent.MOUSE_UP, release, false, 0, true);
		} else {
			thumb.removeEventListener(MouseEvent.MOUSE_OVER, rollOver);
			thumb.removeEventListener(MouseEvent.MOUSE_OUT, rollOut);
			thumb.removeEventListener(MouseEvent.MOUSE_DOWN, press);
			thumb.removeEventListener(MouseEvent.MOUSE_UP, release);
		}
	}
	public function reset():void {
		thumb.x = _width*(initValue-min)/(max-min);
		value = initValue;
		txt.text = String(value);
	}
	private function createGrid(target:Sprite):void {
		for (var n:uint = 0; n <= grid; n++) {
			var w:uint = Math.floor(_width/grid);
			if (n == 0 || n == grid*0.5 || n == grid) {
				createGridLine(target, w*n, mColor);
				var _txt:TextField = new TextField();
				target.addChild(_txt);
				_txt.x = w*n - 20;
				_txt.y = 13;
				_txt.width = 40;
				_txt.height = 14;
				_txt.selectable = false;
				//_txt.embedFonts = true;
				//_txt.antiAliasType = AntiAliasType.ADVANCED;
				//_txt.antiAliasType = AntiAliasType.NORMAL;
				_txt.textColor = mColor;
				var tfc:TextFormat = new TextFormat();
				tfc.font = fontType;
				tfc.size = 8;
				tfc.align = TextFormatAlign.CENTER;
				_txt.defaultTextFormat = tfc;
				if (n == 0) _txt.text = String(min);
				if (n == grid*0.5) _txt.text = String(min+(max-min)*0.5);
				if (n == grid) _txt.text = String(max);
			} else {
				createGridLine(target, w*n, gColor);
			}
		}
	}
	private function createThumb(target:Shape, w:uint, h:uint, y:uint, color:uint, alpha:Number = 1):void {
		target.graphics.beginFill(color, alpha);
		target.graphics.drawRoundRect(-w*0.5, -y, w, h, w);
		target.graphics.endFill();
	}
	private function createGradientHole(target:Shape, w:uint, c:Number):void {
		var colors:Array = [0x000000, 0x000000];
		var alphas:Array = [0.4, 0];
		var ratios:Array = [0, 255];
		var matrix:Matrix = new Matrix();
		matrix.createGradientBox(w+c*2, c*2, 0.5*Math.PI, -c, -c);
		target.graphics.beginGradientFill(GradientType.LINEAR, colors, alphas, ratios, matrix, SpreadMethod.PAD, InterpolationMethod.RGB, 0);
		target.graphics.drawRoundRect(-c, -c, w+c*2, c*2, c*2);
		target.graphics.endFill();
	}
	private function createGridLine(target:Sprite, x:uint, color:uint):void {
		target.graphics.lineStyle(0, color);
		target.graphics.moveTo(x, 8);
		target.graphics.lineTo(x, 12);
	}

}


//////////////////////////////////////////////////
//	 Wheelクラス
//////////////////////////////////////////////////

import flash.display.Sprite;
import flash.display.Shape;
import flash.text.TextField;
import flash.text.TextFieldType;
import flash.text.AntiAliasType;
import flash.text.TextFieldAutoSize;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;
import flash.filters.DropShadowFilter;
import flash.geom.ColorTransform;
import flash.events.Event;
import flash.events.MouseEvent;

class Wheel extends Sprite {
	private var base:Sprite;
	private var thumb:Sprite;
	private var point:Shape;
	private var hit:Shape;
	private var title:TextField;
	private var txt:TextField;
	private var label:String = "";
	private static var fontType:String = "_ゴシック";
	private static var __width:uint = 90;
	private static var _height:uint = 85;
	private static var tHeight:uint = 20;
	private static var bColor:uint = 0xFFFFFF;
	private static var cColor:uint = 0x999999;
	private static var sColor:uint = 0x000000;
	private static var tColor:uint = 0x666666;
	private static var pColor:uint = 0x666666;
	private static var offColor:uint = 0xCCCCCC;
	private static var cColorTrans:ColorTransform;
	private static var pColorTrans:ColorTransform;
	private static var offColorTrans:ColorTransform;
	private var zero:Number = 0;
	private var angle:Number = 0;
	private var shade:DropShadowFilter;
	private var initValue:Number;
	private var value:Number;
	private var _enabled:Boolean = true;

	public function Wheel() {
	}

	public function init(option:Object):void {
		if (option.label != undefined) label = option.label;
		if (option.zero != undefined) zero = option.zero;
		if (option.angle != undefined) angle = option.angle;
		value = initValue = angle;
		draw();
	}
	private function draw():void {
		shade = new DropShadowFilter(1, 90, sColor, 0.4, 4, 4, 2, 3, false, false);
		cColorTrans = new ColorTransform();
		cColorTrans.color = cColor;
		pColorTrans = new ColorTransform();
		pColorTrans.color = pColor;
		offColorTrans = new ColorTransform();
		offColorTrans.color = offColor;
		base = new Sprite();
		thumb = new Sprite();
		point = new Shape();
		hit = new Shape();
		title = new TextField();
		txt = new TextField();
		addChild(base);
		base.addChild(thumb);
		thumb.addChild(point);
		thumb.addChild(hit);
		addChild(title);
		addChild(txt);
		addChild(thumb);
		base.x = thumb.x = 50;
		base.y = thumb.y = 55;
		createDonut(base, 30, 10);
		base.filters = [shade];
		point.x = 20;
		createCircle(point, 5, bColor, 1);
		hit.x = 20;
		createCircle(hit, 10, bColor, 0);
		title.height = tHeight-1;
		title.type = TextFieldType.DYNAMIC;
		title.selectable = false;
		//title.embedFonts = true;
		//title.antiAliasType = AntiAliasType.ADVANCED;
		title.textColor = tColor;
		title.autoSize = TextFieldAutoSize.LEFT;
		var tfl:TextFormat = new TextFormat();
		tfl.font = fontType;
		tfl.size = 12;
		tfl.align = TextFormatAlign.LEFT;
		title.defaultTextFormat = tfl;
		title.text = label;
		txt.x = 50;
		txt.width = 40;
		txt.height = tHeight-1;
		txt.selectable = false;
		//txt.embedFonts = true;
		//txt.antiAliasType = AntiAliasType.ADVANCED;
		var tfr:TextFormat = new TextFormat();
		tfr.font = fontType;
		tfr.size = 12;
		tfr.align = TextFormatAlign.RIGHT;
		txt.defaultTextFormat = tfr;
		reset();
		_up();
		enabled = true;
		thumb.mouseChildren = false;
	}
	private function rollOver(evt:MouseEvent):void {
		_over();
	}
	private function rollOut(evt:MouseEvent):void {
		_up();
	}
	private function press(evt:MouseEvent):void {
		_down();
		thumb.addEventListener(MouseEvent.MOUSE_UP, release, false, 0, true);
		stage.addEventListener(MouseEvent.MOUSE_UP, releaseOutside, false, 0, true);
		stage.addEventListener(MouseEvent.MOUSE_MOVE, change, false, 0, true);
		stage.addEventListener(Event.MOUSE_LEAVE, leave, false, 0, true);
	}
	private function release(evt:MouseEvent):void {
		_up();
		checkValue();
		var e:CompoEvent = new CompoEvent(CompoEvent.SELECT, value);
		dispatchEvent(e);
		thumb.removeEventListener(MouseEvent.MOUSE_UP, release);
		stage.removeEventListener(MouseEvent.MOUSE_UP, releaseOutside);
		stage.removeEventListener(MouseEvent.MOUSE_MOVE, change);
		stage.removeEventListener(Event.MOUSE_LEAVE, leave);
	}
	private function releaseOutside(evt:MouseEvent):void {
		_up();
		checkValue();
		var e:CompoEvent = new CompoEvent(CompoEvent.SELECT, value);
		dispatchEvent(e);
		thumb.removeEventListener(MouseEvent.MOUSE_UP, release);
		stage.removeEventListener(MouseEvent.MOUSE_UP, releaseOutside);
		stage.removeEventListener(MouseEvent.MOUSE_MOVE, change);
		stage.removeEventListener(Event.MOUSE_LEAVE, leave);
	}
	private function leave(evt:Event):void {
		_up();
		checkValue();
		var e:CompoEvent = new CompoEvent(CompoEvent.SELECT, value);
		dispatchEvent(e);
		thumb.removeEventListener(MouseEvent.MOUSE_UP, release);
		stage.removeEventListener(MouseEvent.MOUSE_UP, releaseOutside);
		stage.removeEventListener(MouseEvent.MOUSE_MOVE, change);
		stage.removeEventListener(Event.MOUSE_LEAVE, leave);
	}
	private function _up():void {
		point.transform.colorTransform = cColorTrans;
	}
	private function _over():void {
		point.transform.colorTransform = pColorTrans;
	}
	private function _down():void {
		point.transform.colorTransform = pColorTrans;
	}
	private function _off():void {
		point.transform.colorTransform = offColorTrans;
		txt.textColor = offColor;
	}
	private function change(evt:MouseEvent):void {
		_down();
		thumb.rotation = Math.round(Math.atan2(base.mouseY, base.mouseX)/Math.PI*180);
		evt.updateAfterEvent();
		checkValue();
		var e:CompoEvent = new CompoEvent(CompoEvent.CHANGE, value);
		dispatchEvent(e);
	}
	private function checkValue():void {
		value = (thumb.rotation + 360 + 90 - zero)%360;
		txt.text = String(value);
	}
	public function get enabled():Boolean {
		return _enabled;
	}
	public function set enabled(param:Boolean):void {
		_enabled = param;
		if (!_enabled) _off();
		thumb.buttonMode = _enabled;
		thumb.mouseEnabled = _enabled;
		thumb.useHandCursor = _enabled;
		if (_enabled) {
			thumb.addEventListener(MouseEvent.MOUSE_OVER, rollOver, false, 0, true);
			thumb.addEventListener(MouseEvent.MOUSE_OUT, rollOut, false, 0, true);
			thumb.addEventListener(MouseEvent.MOUSE_DOWN, press, false, 0, true);
		} else {
			thumb.removeEventListener(MouseEvent.MOUSE_OVER, rollOver);
			thumb.removeEventListener(MouseEvent.MOUSE_OUT, rollOut);
			thumb.removeEventListener(MouseEvent.MOUSE_DOWN, press);
		}
	}
	public function reset():void {
		value = initValue;
		thumb.rotation = value - 90 + zero;
		txt.text = String(value);
	}
	private function createCircle(target:Shape, r:uint, color:uint, alpha:Number):void {
		target.graphics.beginFill(color, alpha);
		target.graphics.drawCircle(0, 0, r);
		target.graphics.endFill();
	}
	private function createDonut(target:Sprite, outer:uint, inner:uint):void {
		target.graphics.beginFill(bColor);
		target.graphics.drawCircle(0, 0, outer);
		target.graphics.drawCircle(0, 0, inner);
		target.graphics.endFill();
	}

}


//////////////////////////////////////////////////
// CompoEventクラス
//////////////////////////////////////////////////

import flash.events.Event;

class CompoEvent extends Event {
	public static const SELECT:String = "select";
	public static const CHANGE:String = "change";
	public var value:*;

	public function CompoEvent(type:String, value:*) {
		super(type);
		this.value = value;
	}

	public override function clone():Event {
		return new CompoEvent(type, value);
	}

}


//////////////////////////////////////////////////
// Btnクラス
//////////////////////////////////////////////////

import flash.display.Sprite;
import flash.display.Shape;
import flash.text.TextField;
import flash.text.TextFieldType;
import flash.text.AntiAliasType;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;
import flash.filters.GlowFilter;
import flash.events.MouseEvent;

class Btn extends Sprite {
	public var id:uint;
	private var shade:Shape;
	private var bottom:Shape;
	private var light:Shape;
	private var base:Shape;
	private var txt:TextField;
	private var label:String = "";
	private static var fontType:String = "_ゴシック";
	private var _width:uint = 60;
	private static var _height:uint = 20;
	private static var corner:uint = 5;
	private var type:uint = 1;
	private static var bColor:uint = 0xFFFFFF;
	private static var sColor:uint = 0x000000;
	private static var upColor:uint = 0x666666;
	private static var overColor:uint = 0x333333;
	private static var offColor:uint = 0x999999;
	private static var gColor:uint = 0x0099FF;
	private var blueGlow:GlowFilter;
	private var shadeGlow:GlowFilter;
	private var _clicked:Boolean = false;
	private var _enabled:Boolean = true;

	public function Btn() {
	}

	public function init(option:Object):void {
		if (option.id != undefined) id = option.id;
		if (option.label != undefined) label = option.label;
		if (option.width != undefined) _width = option.width;
		if (option.type != undefined) type = option.type;
		draw();
	}
	private function draw():void {
		switch (type) {
		case 1 :
			bColor = 0xFFFFFF;
			sColor = 0x000000;
			upColor = 0x666666;
			overColor = 0x333333;
			offColor = 0x999999;
			break;
		case 2 :
			bColor = 0x000000;
			sColor = 0xFFFFFF;
			upColor = 0x666666;
			overColor = 0x999999;
			offColor = 0x333333;
			break;
		}
		blueGlow = new GlowFilter(gColor, 0.6, 5, 5, 2, 3, false, true);
		shadeGlow = new GlowFilter(sColor, 0.3, 4, 4, 2, 3, false, true);
		shade = new Shape();
		bottom = new Shape();
		light = new Shape();
		base = new Shape();
		txt = new TextField();
		addChild(shade);
		addChild(bottom);
		addChild(light);
		addChild(base);
		addChild(txt);
		createBase(shade, _width, _height, corner, sColor);
		shade.filters = [shadeGlow];
		createBase(bottom, _width, _height, corner, sColor, 0.3);
		createBase(light, _width, _height, corner, gColor);
		light.filters = [blueGlow];
		createBase(base, _width, _height, corner, bColor);
		txt.x = -_width*0.5;
		txt.y = -_height*0.5;
		txt.width = _width;
		txt.height = _height - 1;
		txt.type = TextFieldType.DYNAMIC;
		txt.selectable = false;
		//txt.embedFonts = true;
		//txt.antiAliasType = AntiAliasType.ADVANCED;
		var tf:TextFormat = new TextFormat();
		tf.font = fontType;
		tf.size = 12;
		tf.align = TextFormatAlign.CENTER;
		txt.defaultTextFormat = tf;
		txt.text = label;
		enabled = true;
		mouseChildren = false;
	}
	private function rollOver(evt:MouseEvent):void {
		_over();
	}
	private function rollOut(evt:MouseEvent):void {
		_up();
	}
	private function press(evt:MouseEvent):void {
		_down();
	}
	private function release(evt:MouseEvent):void {
		_up();
	}
	private function click(evt:MouseEvent):void {
	}
	private function _up():void {
		txt.y = -_height*0.5;
		txt.textColor = upColor;
		base.y = -1;
		light.visible = false;
		light.y = -1;
	}
	private function _over():void {
		txt.y = -_height*0.5;
		txt.textColor = overColor;
		base.y = -1;
		light.visible = true;
		light.y = -1;
	}
	private function _down():void {
		txt.y = -_height*0.5 + 1;
		txt.textColor = overColor;
		base.y = 0;
		light.visible = true;
		light.y = 0;
	}
	private function _off():void {
		txt.y = -_height*0.5 + 1;
		txt.textColor = offColor;
		base.y = 0;
		light.visible = false;
		light.y = 0;
	}
	public function get clicked():Boolean {
		return _clicked;
	}
	public function set clicked(param:Boolean):void {
		_clicked = param;
		enabled = !_clicked;
		if (_clicked) {
			_down();
		} else {
			_up();
		}
	}
	public function get enabled():Boolean {
		return _enabled;
	}
	public function set enabled(param:Boolean):void {
		_enabled = param;
		buttonMode = _enabled;
		mouseEnabled = _enabled;
		useHandCursor = _enabled;
		if (_enabled) {
			_up();
			addEventListener(MouseEvent.MOUSE_OVER, rollOver, false, 0, true);
			addEventListener(MouseEvent.MOUSE_OUT, rollOut, false, 0, true);
			addEventListener(MouseEvent.MOUSE_DOWN, press, false, 0, true);
			addEventListener(MouseEvent.MOUSE_UP, release, false, 0, true);
			addEventListener(MouseEvent.CLICK, click, false, 0, true);
		} else {
			_off();
			removeEventListener(MouseEvent.MOUSE_OVER, rollOver);
			removeEventListener(MouseEvent.MOUSE_OUT, rollOut);
			removeEventListener(MouseEvent.MOUSE_DOWN, press);
			removeEventListener(MouseEvent.MOUSE_UP, release);
			removeEventListener(MouseEvent.CLICK, click);
		}
	}
	private function createBase(target:Shape, w:uint, h:uint, c:uint, color:uint, alpha:Number = 1):void {
		target.graphics.beginFill(color, alpha);
		target.graphics.drawRoundRect(-w*0.5, -h*0.5, w, h, c*2);
		target.graphics.endFill();
	}

}
